@@ -1,7 +1,7 @@
 # __init__.py - main module
 # coding: utf-8
 #
-# Copyright (C) 2014-2016 Arthur de Jong
+# Copyright (C) 2014-2017 Arthur de Jong
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -37,6 +37,15 @@ The following prints all keys, decrypting using a password:
 ...     print('%s %s' % (key.serial, str(key.secret.decode())))
 987654321 12345678901234567890
 
+The following generates an encrypted PSKC file:
+
+>>> pskc = PSKC()
+>>> key = pskc.add_key(
+...     id='456', secret='987654321', manufacturer='Manufacturer',
+...     algorithm = 'urn:ietf:params:xml:ns:keyprov:pskc:hotp')
+>>> pskc.encryption.setup_pbkdf2('passphrase')
+>>> pskc.write('output.pskcxml')
+
 The module should be able to handle most common PSKC files.
 """
 
@@ -45,7 +54,7 @@ __all__ = ['PSKC', '__version__']
 
 
 # the version number of the library
-__version__ = '0.4'
+__version__ = '0.5'
 
 
 class PSKC(object):
@@ -57,68 +66,50 @@ class PSKC(object):
       id: identifier
       encryption: information on used encryption (Encryption instance)
       mac: information on used MAC method (MAC instance)
+      devices: list of devices (Device instances)
       keys: list of keys (Key instances)
     """
 
     def __init__(self, filename=None):
         from pskc.encryption import Encryption
-        from pskc.exceptions import ParseError
         from pskc.mac import MAC
         self.version = None
         self.id = None
         self.encryption = Encryption(self)
         self.mac = MAC(self)
-        self.keys = []
+        self.devices = []
         if filename is not None:
-            from pskc.xml import parse, remove_namespaces
-            try:
-                tree = parse(filename)
-            except Exception:
-                raise ParseError('Error parsing XML')
-            remove_namespaces(tree)
-            self.parse(tree.getroot())
+            from pskc.parser import PSKCParser
+            PSKCParser.parse_file(self, filename)
         else:
             self.version = '1.0'
 
-    def parse(self, container):
-        """Read information from the provided <KeyContainer> tree."""
-        from pskc.exceptions import ParseError
-        from pskc.key import Key
-        from pskc.xml import find, findall
-        if container.tag != 'KeyContainer':
-            raise ParseError('Missing KeyContainer')
-        # the version of the PSKC schema
-        self.version = container.get('Version')
-        if self.version != '1.0':
-            raise ParseError('Unsupported version %r' % self.version)
-        # unique identifier for the container
-        self.id = container.get('Id')
-        # handle EncryptionKey entries
-        self.encryption.parse(find(container, 'EncryptionKey'))
-        # handle MACMethod entries
-        self.mac.parse(find(container, 'MACMethod'))
-        # handle KeyPackage entries
-        for key_package in findall(container, 'KeyPackage'):
-            self.keys.append(Key(self, key_package))
+    @property
+    def keys(self):
+        return tuple(key for device in self.devices for key in device.keys)
 
-    def make_xml(self):
-        from pskc.xml import mk_elem
-        container = mk_elem('pskc:KeyContainer', Version=self.version,
-                            Id=self.id)
-        self.encryption.make_xml(container)
-        self.mac.make_xml(container)
-        for key in self.keys:
-            key.make_xml(container)
-        return container
+    def add_device(self, **kwargs):
+        """Create a new device instance for the PSKC file.
+
+        The device is initialised with properties from the provided keyword
+        arguments if any."""
+        from pskc.device import Device
+        device = Device(self)
+        self.devices.append(device)
+        # assign the kwargs as key properties
+        for k, v in kwargs.items():
+            if not hasattr(device, k):
+                raise AttributeError()
+            setattr(device, k, v)
+        return device
 
     def add_key(self, **kwargs):
         """Create a new key instance for the PSKC file.
 
         The new key is initialised with properties from the provided keyword
         arguments if any."""
-        from pskc.key import Key
-        key = Key(self)
-        self.keys.append(key)
+        device = self.add_device()
+        key = device.add_key()
         # assign the kwargs as key properties
         for k, v in kwargs.items():
             if not hasattr(key, k):
@@ -128,14 +119,9 @@ class PSKC(object):
 
     def write(self, filename):
         """Write the PSKC file to the provided file."""
-        from pskc.xml import tostring
+        from pskc.serialiser import PSKCSerialiser
         if hasattr(filename, 'write'):
-            xml = tostring(self.make_xml())
-            try:
-                filename.write(xml)
-            except TypeError:  # pragma: no cover (Python 3 specific)
-                # fall back to writing as string for Python 3
-                filename.write(xml.decode('utf-8'))
+            PSKCSerialiser.serialise_file(self, filename)
         else:
             with open(filename, 'wb') as output:
                 self.write(output)
