@@ -29,7 +29,6 @@ with the PSKC encryption key.
 """
 
 
-import base64
 import re
 
 
@@ -53,6 +52,18 @@ def get_hmac(algorithm):
         return lambda key, value: hmac.new(key, value, digestmod).digest()
 
 
+def get_mac(algorithm, key, value):
+    """Generate the MAC value over the specified value."""
+    from pskc.exceptions import DecryptionError
+    if algorithm is None:
+        raise DecryptionError('No MAC algorithm set')
+    hmacfn = get_hmac(algorithm)
+    if hmacfn is None:
+        raise DecryptionError(
+            'Unsupported MAC algorithm: %r' % algorithm)
+    return hmacfn(key, value)
+
+
 class MAC(object):
     """Class describing the MAC algorithm to use and how to get the key.
 
@@ -69,41 +80,6 @@ class MAC(object):
         self.key_cipher_value = None
         self.key_algorithm = None
 
-    def parse(self, mac_method):
-        """Read MAC information from the <MACMethod> XML tree."""
-        from pskc.xml import find, findtext, findbin
-        if mac_method is None:
-            return
-        self.algorithm = mac_method.get('Algorithm')
-        mac_key = find(mac_method, 'MACKey')
-        if mac_key is not None:
-            self.key_cipher_value = findbin(mac_key, 'CipherData/CipherValue')
-            encryption_method = find(mac_key, 'EncryptionMethod')
-            if encryption_method is not None:
-                self.key_algorithm = encryption_method.attrib.get('Algorithm')
-        mac_key_reference = findtext(mac_method, 'MACKeyReference')
-
-    def make_xml(self, container):
-        from pskc.xml import mk_elem
-        if not self.algorithm and not self.key:
-            return
-        mac_method = mk_elem(
-            container, 'pskc:MACMethod', Algorithm=self.algorithm, empty=True)
-        mac_key = mk_elem(mac_method, 'pskc:MACKey', empty=True)
-        mk_elem(
-            mac_key, 'xenc:EncryptionMethod',
-            Algorithm=self.pskc.encryption.algorithm)
-        cipher_data = mk_elem(mac_key, 'xenc:CipherData', empty=True)
-        if self.key_cipher_value:
-            mk_elem(
-                cipher_data, 'xenc:CipherValue',
-                base64.b64encode(self.key_cipher_value).decode())
-        elif self.key_plain_value:
-            mk_elem(
-                cipher_data, 'xenc:CipherValue', base64.b64encode(
-                    self.pskc.encryption.encrypt_value(self.key_plain_value)
-                ).decode())
-
     @property
     def key(self):
         """Provides access to the MAC key binary value if available."""
@@ -112,6 +88,8 @@ class MAC(object):
         elif self.key_cipher_value:
             return self.pskc.encryption.decrypt_value(
                 self.key_cipher_value, self.key_algorithm)
+        # fall back to encryption key
+        return self.pskc.encryption.key
 
     @key.setter
     def key(self, value):
@@ -126,7 +104,7 @@ class MAC(object):
 
     @algorithm.setter
     def algorithm(self, value):
-        from pskc.encryption import normalise_algorithm
+        from pskc.algorithms import normalise_algorithm
         self._algorithm = normalise_algorithm(value)
 
     @property
@@ -142,26 +120,7 @@ class MAC(object):
 
     def generate_mac(self, value):
         """Generate the MAC over the specified value."""
-        from pskc.exceptions import DecryptionError
-        key = self.key
-        if key is None:
-            raise DecryptionError('No MAC key available')
-        hmacfn = get_hmac(self.algorithm)
-        if hmacfn is None:
-            raise DecryptionError(
-                'Unsupported MAC algorithm: %r' % self.algorithm)
-        return hmacfn(key, value)
-
-    def check_value(self, value, value_mac):
-        """Check if the provided value matches the MAC.
-
-        This will return None if there is no MAC to be checked. It will
-        return True if the MAC matches and raise an exception if it fails.
-        """
-        from pskc.exceptions import DecryptionError
-        if self.generate_mac(value) != value_mac:
-            raise DecryptionError('MAC value does not match')
-        return True
+        return get_mac(self.algorithm, self.key, value)
 
     def setup(self, key=None, algorithm=None):
         """Configure an encrypted MAC key.
