@@ -99,7 +99,7 @@ class Policy(object):
     # The PIN is used in the algorithm computation.
     PIN_USE_ALGORITHMIC = 'Algorithmic'
 
-    def __init__(self, key=None, policy=None):
+    def __init__(self, key=None):
         """Create a new policy, optionally linked to the key and parsed."""
         self.key = key
         self.start_date = None
@@ -113,72 +113,6 @@ class Policy(object):
         self.pin_max_length = None
         self.pin_encoding = None
         self.unknown_policy_elements = False
-        self.parse(policy)
-
-    def parse(self, policy):
-        """Read key policy information from the provided <Policy> tree."""
-        from pskc.xml import (
-            find, findall, findtext, findint, findtime, getint)
-        if policy is None:
-            return
-
-        self.start_date = findtime(policy, 'StartDate')
-        self.expiry_date = findtime(policy, 'ExpiryDate')
-        self.number_of_transactions = findint(
-            policy, 'NumberOfTransactions')
-        for key_usage in findall(policy, 'KeyUsage'):
-            self.key_usage.append(findtext(key_usage, '.'))
-
-        pin_policy = find(policy, 'PINPolicy')
-        if pin_policy is not None:
-            self.pin_key_id = pin_policy.get('PINKeyId')
-            self.pin_usage = pin_policy.get('PINUsageMode')
-            self.pin_max_failed_attemtps = getint(
-                pin_policy, 'MaxFailedAttempts')
-            self.pin_min_length = getint(pin_policy, 'MinLength')
-            self.pin_max_length = getint(pin_policy, 'MaxLength')
-            self.pin_encoding = pin_policy.get('PINEncoding')
-            # check for child elements
-            if list(pin_policy):
-                self.unknown_policy_elements = True
-            # check for unknown attributes
-            known_attributes = set([
-                'PINKeyId', 'PINUsageMode', 'MaxFailedAttempts', 'MinLength',
-                'MaxLength', 'PINEncoding'])
-            if set(pin_policy.keys()) - known_attributes:
-                self.unknown_policy_elements = True
-
-        # check for other child elements
-        known_children = set([
-            'StartDate', 'ExpiryDate', 'NumberOfTransactions', 'KeyUsage',
-            'PINPolicy'])
-        for child in policy:
-            if child.tag not in known_children:
-                self.unknown_policy_elements = True
-
-    def make_xml(self, key):
-        from pskc.xml import mk_elem
-        # check if any policy attribute is set
-        if not self.key_usage and all(x is None for x in (
-                self.start_date, self.expiry_date,
-                self.number_of_transactions, self.pin_key_id, self.pin_usage,
-                self.pin_max_failed_attemtps, self.pin_min_length,
-                self.pin_max_length, self.pin_encoding)):
-            return
-        policy = mk_elem(key, 'pskc:Policy', empty=True)
-        mk_elem(policy, 'pskc:StartDate', self.start_date)
-        mk_elem(policy, 'pskc:ExpiryDate', self.expiry_date)
-        mk_elem(policy, 'pskc:PINPolicy',
-                PINKeyId=self.pin_key_id,
-                PINUsageMode=self.pin_usage,
-                MaxFailedAttempts=self.pin_max_failed_attemtps,
-                MinLength=self.pin_min_length,
-                MaxLength=self.pin_max_length,
-                PINEncoding=self.pin_encoding)
-        for usage in self.key_usage:
-            mk_elem(policy, 'pskc:KeyUsage', usage)
-        mk_elem(policy, 'pskc:NumberOfTransactions',
-                self.number_of_transactions)
 
     def may_use(self, usage=None, now=None):
         """Check whether the key may be used for the provided purpose."""
@@ -203,8 +137,8 @@ class Policy(object):
     @property
     def pin_key(self):
         """Reference to the PSKC Key that holds the PIN (if any)."""
-        if self.pin_key_id and self.key and self.key.pskc:
-            for key in self.key.pskc.keys:
+        if self.pin_key_id and self.key and self.key.device.pskc:
+            for key in self.key.device.pskc.keys:
                 if key.id == self.pin_key_id:
                     return key
 
